#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
# Convert Line Breaks
# Version: 1.1.2
# © 2011 Red Sweater Software, All Rights Reserved.
#
# This script works as follows:
#
#	1. Glom all the "paragraph-like chunks" into a python array
#	2. Pass through the array of chunks, artificially merging any exceptions
#	   that should not actually be treated as separable paragraphs (e.g. whitespace
#	   that occurs within <pre> or <code> blocks.
#	3. For each chunk that doesn't appear to already be an intentionally marked-up
#	   chunk of HTML, add artificial <p> tags to turn them into paragraphs.
#

import sys
import string

inputLines = sys.stdin.readlines()
inputGroups = []
skippingBlanks = True

def isBlankLine(testLine):
	return (testLine.rstrip() == "")

# 1. This is the glomming into chunks part
for thisLine in inputLines:
	# Non-blank line? If we've been skipping
	# blanks, looking for valid blocks, then start a 
	# new group	
	if (isBlankLine(thisLine) == False):
		# We only strip trailing whitespace to preserve indentation
		canonicalLine = thisLine.rstrip() + "\n"
		
		if (len(inputGroups) == 0):
			inputGroups += [canonicalLine,]
		else:
			inputGroups[-1] += canonicalLine
	else:
		# Got a blank, which, for now we just put in as its own group so
		# it will be preserved verbatim later
		inputGroups += [thisLine,]

#print inputGroups
# 2. Now, for each chunk we have, if it has a <pre> or a <code> that is not matched, 
# 	 we're going to join with the following chunk until we have a closing </pre> and/or </code>
mergedInputGroups = []
openPreCount = 0
openCodeCount = 0
for thisBlock in inputGroups:
	# if the running open pre or open code count is > 0, then we 
	# are currently sucking up blocks into the last block in the list
	if (openPreCount > 0) or (openCodeCount > 0):
		if (len(mergedInputGroups) == 0):
			mergedInputGroups += [thisBlock,]
		else:
			mergedInputGroups[-1] += thisBlock
	else:
		mergedInputGroups += [thisBlock,]

	openPreCount += thisBlock.count("<pre>")
	openPreCount -= thisBlock.count("</pre>")
	openCodeCount += thisBlock.count("<code>")
	openCodeCount -= thisBlock.count("</code>")

# Now for the output, we print out the non-blank lines surrounded by p tags,
# if appropriate, and print out the blanks verbatim
firstBlock = True
for thisBlock in mergedInputGroups:

	if (isBlankLine(thisBlock)):
		sys.stdout.write("\n")
	else:		
		if (firstBlock == False):
			sys.stdout.write("\n\n")
		firstBlock = False
		thisBlock = thisBlock.strip()
		
		# We add paragraph tags fairly conservatively: only if the 
		# block we are looking at does not appear to be "wrapped" in any form
		# of HTML markup tags already. Note, it might be better to 
		# look more specifically for evidence of "block level" markup, but 
		# for now we just concede that if it looks wrapped, the user knows what they are doing.
		shouldAddParagraphTags = False
		if ((thisBlock.startswith("<") == False) or (thisBlock.endswith(">") == False)):
			shouldAddParagraphTags = True
			
		if (shouldAddParagraphTags):
			sys.stdout.write("<p>")

		sys.stdout.write(thisBlock)
		
		if (shouldAddParagraphTags):
			sys.stdout.write("</p>")
